@@ -11,9 +11,12 @@ checkforcings <- function (forcings, times, dllname,
 
  if (is.null(initforc))
    stop(paste("initforc should be loaded if there are forcing functions ",initforc))
- if (is.loaded(initforc, PACKAGE = dllname, type = "") ||
+
+ if (class (initforc) == "CFunc") {
+      ModelForc <- body(initforc)[[2]]
+ }  else if (is.loaded(initforc, PACKAGE = dllname, type = "") ||
      is.loaded(initforc, PACKAGE = dllname, type = "Fortran")) {
- ModelForc <- getNativeSymbolInfo(initforc, PACKAGE = dllname)$address
+   ModelForc <- getNativeSymbolInfo(initforc, PACKAGE = dllname)$address
  } else
    stop(paste("initforc should be loaded if there are forcing functions ",initforc))
 
